#!/usr/bin/env python

#
# Python node to read Nao's sensors
# This code is currently compatible to NaoQI version 1.12.3
#
# Copyright 2012 - 2014 Moises Martinez, University Carlos III
#			2014 Marcos Zimmermann Casado, University Carlos III
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    # Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#    # Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    # Neither the name of the Unversity Carlos III nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import roslib

roslib.load_manifest('nao_core')

import rospy

from nao_node import *
from std_msgs.msg import Bool
from std_msgs.msg import String

class naoSensors(naoNode):
	def __init__(self, name, ip, port, robotIp, robotPort): 

		naoNode.__init__(self, name, ip, port, robotIp, robotPort)
		
		self.names = ['DCM/Time',
						'Device/SubDeviceList/InertialSensor/AngleX/Sensor/Value',
						'Device/SubDeviceList/InertialSensor/AngleY/Sensor/Value',
						'Device/SubDeviceList/InertialSensor/GyrX/Sensor/Value',
						'Device/SubDeviceList/InertialSensor/GyrY/Sensor/Value',
						'Device/SubDeviceList/InertialSensor/AccX/Sensor/Value', 
						'Device/SubDeviceList/InertialSensor/AccY/Sensor/Value',
						'Device/SubDeviceList/InertialSensor/AccZ/Sensor/Value',
						'Device/SubDeviceList/RHand/Position/Sensor/Value',
						'Device/SubDeviceList/LHand/Position/Sensor/Value',
						'Device/SubDeviceList/Battery/Charge/Sensor/Status',
						'Device/SubDeviceList/US/Left/Sensor/Value',
						'Device/SubDeviceList/US/Left/Sensor/Value1',
						'Device/SubDeviceList/US/Left/Sensor/Value2',
						'Device/SubDeviceList/US/Left/Sensor/Value3',
						'Device/SubDeviceList/US/Left/Sensor/Value4',
						'Device/SubDeviceList/US/Left/Sensor/Value5',
						'Device/SubDeviceList/US/Left/Sensor/Value6',
						'Device/SubDeviceList/US/Left/Sensor/Value7',
						'Device/SubDeviceList/US/Right/Sensor/Value',
						'Device/SubDeviceList/US/Right/Sensor/Value1',
						'Device/SubDeviceList/US/Right/Sensor/Value2',
						'Device/SubDeviceList/US/Right/Sensor/Value3',
						'Device/SubDeviceList/US/Right/Sensor/Value4',
						'Device/SubDeviceList/US/Right/Sensor/Value5',
						'Device/SubDeviceList/US/Right/Sensor/Value6',
						'Device/SubDeviceList/US/Right/Sensor/Value7']

		self.rate = 1.0
		self.sensorLevel = 4
		self.name = name
		self.robotName = 'rover0'
		self.x = 0
		self.y = 0

		self.motionProxy = self.connectProxy('ALMotion')
		self.memoryProxy = self.connectProxy('ALMemory')
		#self.poseProxy = self.connectProxy('ALPose')

		if (self.memoryProxy is None) or (self.motionProxy is None):
			rospy.logerror('Could not get a proxy to ALMemory')
			exit(1)

		#if (self.poseProxy is None):
		#	rospy.logerror('Could not get a proxy to ALPose')
		#	self.pose = False
		#else:
		#	self.pose = True

		self.sensor_pub = rospy.Publisher('nao_sensors_xml', String)

		rospy.init_node(self.name)

	def generateXML(self):

		values = self.memoryProxy.getListData(self.names)
		angles_names  = self.motionProxy.getJointNames('Body')
		angles_values = self.motionProxy.getAngles('Body', False)

		buffer  = '<atom robot=\'%s\'/>' % self.robotName;
		buffer += '<atom predicate=\'x\'>%s</atom>' % self.x;
		buffer += '<atom predicate=\'y\'>%s</atom>' % self.y;

		#if (self.pose):
		#	robot_pose = self.poseProxy.getActualPoseAndTime()
		#	buffer += '<atom predicate=\'pose\'>%s</atom>' % robot_pose[0];

		buffer += '<atom predicate=\'battery\'>%s</atom>' % values[8]
		buffer += '<atom predicate=\'leftHand\'>%s</atom>' % values[9]
		buffer += '<atom predicate=\'rightHand\'>%s</atom>' % values[10]

		buffer += '<atom predicate=\'Gyroscope\'>'
		buffer += '<term name=\'AngleX\'>%s</term>' % values[1]
		buffer += '<term name=\'AngleY\'>%s</term>' % values[2]
		buffer += '<term name=\'GyroX\'>%s</term>' % values[3]
		buffer += '<term name=\'GyroY\'>%s</term>' % values[4]
		buffer += '</atom>'

		buffer += '<atom predicate=\'Accelerometer\'>'
		buffer += '<term name=\'X\'>%s</term>' % values[5]
		buffer += '<term name=\'Y\'>%s</term>' % values[6]
		buffer += '<term name=\'Z\'>%s</term>' % values[7]
		buffer += '</atom>'

		buffer += '<atom predicate=\'Sonar\'>'
		
		for i in range(11, 19):
			buffer += '<term name=\'sonar_left_%d\'>%s</term>' % ((i-11), values[i])

		for i in range(19, 26):
			buffer += '<term name=\'sonar_right_%d\'>%s</term>' % ((i-19), values[i])

		buffer += '</atom>'

		buffer += '<atom predicate=\'Body\'>'

		for i in range(len(angles_names)):
			buffer += '<term name=\'%s\'>%s</term>' % (angles_names[i], angles_values[i])

		buffer += '</atom>'

		self.sensor_pub.publish(String(buffer))

	def run(self):
		while not rospy.is_shutdown():
			self.generateXML()
			rospy.sleep(self.rate)

	def stop(self):
		if (self.memoryProxy != None):
			self.memoryProxy.unsubscribe()
		if (self.motionProxy != None):
			self.motionProxy.unsubscribe()

if __name__ == '__main__':
	try:
		sensors = naoSensors('nao_sensors', 'localhost', 10510, '192.168.1.100', 9559)
		sensors.run()
		
	except rospy.ROSInterruptException: pass
