# -*- encoding: UTF-8 -*-
#
# This is a tiny example that shows how to show live images from Nao using PyQt.
# You must have python-qt4 installed on your system.
#
from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import QWidget, QImage, QPainter
from vision import ColorTracker
# To get the constants relative to the video.
import vision_definitions

import Image

class ImageWidget(QtGui.QDialog):
    """
    Tiny widget to display camera images from Naoqi.
    """
    def __init__(self, videoProxy, imageClient, parent=None):
        """
        Initialization.
        """
        QtGui.QDialog.__init__(self, parent)
        #QWidget.__init__(self, parent)
        self._image = QImage()
        self.setWindowTitle('Nao')
        
        self._imgWidth = 320
        self._imgHeight = 240
        self.resize(self._imgWidth, self._imgHeight)

        # Show grid
        self._grid = False
        self._calibrationCircle = False
        self._circleRadius = None
        self._pen = None
        
        self._pen = QtGui.QPen(QtCore.Qt.red, 1, QtCore.Qt.SolidLine)
        
        #color filtering
        self._ct = ColorTracker.ColorTracker()
        self._filter = False
        self._minH = None
        self._maxH = None
        
        # Proxy to ALVideoDevice.
        self._videoProxy = videoProxy

        # Our video module name.
        self._imgClient = imageClient

        # This will contain this alImage we get from Nao.
        self._alImage = None

        self._PILImage = None
        
        self._updateImage()
        # Trigget 'timerEvent' every 100 ms.
        self.startTimer(100)
        

    def getImage(self):
        return self._alImage
    
    
    def getQImage(self):
        return self._image
    
    
    def getPILImage(self):
        self._PILImage = Image.fromstring("RGB", (self._alImage[0], self._alImage[1]), self._alImage[6])
        return self._PILImage


    def filter(self, minH, maxH):
        self._filter = True
        self._minH = minH
        self._maxH = maxH


    def _unregisterImageClient(self):
        """
        Unregister our naoqi video module.
        """
        self._videoProxy.unsubscribe(self._imgClient)
            
            
    def showGrid(self):
        self._grid = True
    
    
    def showCalibrationCircle(self, radius):
        self._calibrationCircle = True
        self._circleRadius = radius
     
        
    def printGrid(self, painter):
        """
        width:320 height:240 
        """
        painter.setPen(self._pen)
        painter.drawLine(0, self._imgHeight/2, self._imgWidth, self._imgHeight/2)  
        painter.drawLine(self._imgWidth/4, 0, self._imgWidth/4, self._imgHeight)  
        painter.drawLine(self._imgWidth/2, 0, self._imgWidth/2, self._imgHeight)  
        painter.drawLine(self._imgWidth*3/4, 0, self._imgWidth*3/4, self._imgHeight)  
        #painter.drawLine()

    def printCalibrationCircle(self, painter):
        
        painter.setPen(self._pen)
        center = QtCore.QPoint(self._imgWidth/2, self._imgHeight/2)
        painter.drawEllipse(center, self._circleRadius, self._circleRadius)
        
          
    def circumferencePoints(self,minX, maxX, y, grid, painter):
        for x in range (minX, maxX+1):
            painter.drawPoint(x, y)
            grid.append(x)
        
        return grid
    
            
    def paintEvent(self, event):
        """
        Draw the QImage on screen. 
        """
        painter = QPainter(self)
        
        painter.drawImage(painter.viewport(), self._image)

        if(self._grid):
            self.printGrid(painter)

        if(self._calibrationCircle):
            self.printCalibrationCircle(painter)

    def _updateImage(self):
        """
        Retrieve a new image from Nao.
        """
        self._alImage = self._videoProxy.getImageRemote(self._imgClient)
        self._image = QImage(self._alImage[6],           # Pixel array.
                             self._alImage[0],           # Width.
                             self._alImage[1],           # Height.
                             QImage.Format_RGB888)
        #print self._alImage

    def timerEvent(self, event):
        """
        Called periodically. Retrieve a nao image, and update the widget.
        """
        self._updateImage()
        if self._filter:
            imgArray = self._ct.colorFilter(self._minH, self._maxH, self.getPILImage())
            self._image= QImage(imgArray,           # Pixel array.
                             self._alImage[0],           # Width.
                             self._alImage[1],           # Height.
                             QImage.Format_RGB888)
        self.update()


    def __del__(self):
        """
        When the widget is deleted, we unregister our naoqi video module.
        """
        self._unregisterImageClient()
